function IcuCtrl($scope, mapService) {
    $scope.message = '';
    $scope.map = null;
    $scope.initData = {};

    $scope.users = [];
    $scope.hints = [];
    $scope.topJoins = [];

    // chat stuff here, need to go into service
    $scope.chatBuddyMessages = [];
    $scope.chatBuddy = { un: 'no one', uuid: null };
    $scope.firstPublish = true;
    $scope.newMessageText = '';

    $scope.myUser = {
        username: localStorage.getItem('username') !== null ? localStorage.getItem('username') : 'new user',
        pin: null,
        uuid: localStorage.getItem('uuid'),
        keyword: '',
        latitude: 0,
        longitude: 0,

        getJoinPostData: function() {
            return {
                uuid: this.uuid,
                keyword: this.keyword,
                latitude: this.latitude,
                longitude: this.longitude
            };
        },

        getPrivateJoinPostData: function() {
            return {
                uuid: this.uuid,
                keyword: this.keyword,
                latitude: this.latitude,
                longitude: this.longitude,
                'private': true
                // private is written as string because private is reserved word in javascript
            };
        }
    };

    /**
     * Init data from /start.json resource
     */
    (function() {
        $.get(SERVER + '/start.json', function(data) {
            $scope.initData = data;
            $scope.$apply(function(){
                $scope.topJoins = data.top_joins;
            });
        }, 'json');

        $scope.map = mapService.initMap($scope.myUser);
    }());

    $scope.join = function(joinData) {
        if (!validateJoin(joinData)) {
            return false;
        }

        var jsonPostString = JSON.stringify(joinData),
            joinUri = SERVER + $scope.initData.sitemap.join.uri;

        console.log('Join with |' + jsonPostString);
        subscribeToJoinChannel($scope.myUser.keyword);

        $.post(joinUri, jsonPostString, function(data) {
            if (data.my_user === undefined) {
                return false;
            }

            saveUserLocallyAndSubscribeToChat(data);

            if (data.radius >= 0 && data.users !== undefined && data.users.length > 0) {
                console.log('Join returned: ' + data.users.length + ' users. -> ');
                console.log(data);

                $scope.$apply(function(){
                    $scope.users = data.users;
                    $scope.message = '';
                });

                mapService.addUsersToMap($scope.map, $scope.myUser, data.users, data.radius);
            } else {

                $scope.$apply(function(){
                    $scope.message = NO_USERS_FOUND;
                    $scope.users = [];
                    $scope.hints = data.hints;
                });

                mapService.clearUserPins($scope.map);
                mapService.clearCircle($scope.map);
            }

            // publish this join
            publishToPubNub($scope.myUser.keyword, getMyUserAsJson());
        },
        'json');
    };

    $scope.setKeywordAndJoin = function(keyword) {
        $scope.myUser.keyword = keyword;
        $scope.join($scope.myUser.getJoinPostData());
    };

    $scope.showUserOnMap = function(user) {
        $.mobile.changePage('#map', 'slide');
    };

    $scope.chatWith = function(user) {
        $scope.chatBuddy = user;
        $.mobile.changePage('#chat', 'slide');
    };

    function validateJoin(joinData) {
        if (joinData.keyword.length < 2) {
            $scope.message = KEYWORD_TOO_SHORT;
            return false;
        }

        if (joinData.longitude === 0 || joinData.latitude === 0) {
            $scope.message = CANT_GET_LOCATION;
            return false;
        }

        return true;
    }

    function saveUserLocallyAndSubscribeToChat(data) {
        if (localStorage.getItem('uuid') === null || localStorage.getItem('uuid') !== data.my_user.uuid) {

            localStorage.setItem('uuid', data.my_user.uuid);
            $scope.myUser.uuid = data.my_user.uuid;

            localStorage.setItem('username', data.my_user.un);
            $scope.myUser.username = data.my_user.un;
        }

        // subscribe on your own uuid to listen for incoming chat (request) messages
        $scope.subscribeToChat($scope.myUser.uuid);
    }

    function getMyUserAsJson() {
        var myUser = {
            user: {
                un: $scope.myUser.username,
                uuid: $scope.myUser.uuid,
                ll: {
                    lat: $scope.myUser.latitude,
                    lng: $scope.myUser.longitude
                }
            }
        };
        return JSON.stringify(myUser);
    }

    function subscribeToJoinChannel(keyword) {
        pubnub.subscribe({
            channel: keyword,
            callback: function(message) {
                // another user joined the keyword
                var data = JSON.parse(message),
                    user = data.user;

                // don't show yourself
                if (data.user.un !== $scope.myUser.username) {
                    var radius = distVincenty(
                        $scope.myUser.latitude,
                        $scope.myUser.longitude,
                        user.ll.lat,
                        user.ll.lng);

                    mapService.addUsersToMap(
                        $scope.map,
                        $scope.myUser,
                        $.makeArray(user),
                        parseInt(radius, 10) + 200);

                    // PhoneGap
                    navigator.notification.beep(1);
                }
            },
            connect: function() {
                console.log("Join pubsub connected.");
            },
            error: function(e) {
                console.log(e);
            }
        });
    }

    function publishToPubNub(keyword, message) {
        pubnub.publish({
            channel: keyword,
            message: message
        });
    }

    $scope.subscribeToChat = function(channel) {
        pubnub.subscribe({
            channel: channel,
            restore: false,

            callback: function(userAndMessage) {
                // chat init request
                if (userAndMessage.init === 'true') {
                    // PhoneGap
                    navigator.notification.beep(2);

                    var chatAccept = confirm(INCOMING_MESSAGE + ' '  + userAndMessage.un),
                        newChannel = $scope.getChannelFromUuids();

                    if (chatAccept) {
                        $scope.firstPublish = false;
                        $scope.chatBuddy.un = userAndMessage.un;
                        $scope.chatBuddy.uuid = userAndMessage.uuid;
                        $scope.subscribeToChat(newChannel);
                        $.mobile.changePage('#chat', 'slide');
                    }
                }

                $scope.$apply(function(){
                    $scope.chatBuddyMessages.push(userAndMessage);
                });
            },

            connect: function() {
                if (channel.indexOf(':') === -1) {
                    console.log('Successfully connected on my own channel for incoming requests: ' + channel);
                } else {
                    console.log('Successfully connected on channel for chat: ' + channel);
                    $scope.$apply(function(){
                        //load the message history from PubNub
                        $scope.history(channel);
                    });
                }
            }
        });
    };

    //gets the messages history
    $scope.history = function(channel) {
        pubnub.history( {
            channel : channel,
            limit   : 10
        }, function(messages) {
            // Shows All Messages within the limit
            $scope.$apply(function(){
                for (var i = messages.length - 1; i >= 0; i--) {
                    messages[i].oldClass = 'old';
                    $scope.chatBuddyMessages.unshift(messages[i]);
                }
            });
        });
    };

    $scope.getChannelFromUuids = function() {
        var myUuid = $scope.myUser.uuid;
        var buddyUuid = $scope.chatBuddy.uuid;
        var firstSmallUuid = myUuid < buddyUuid ? myUuid : buddyUuid;
        var secondBigUuid = myUuid > buddyUuid ? myUuid : buddyUuid;
        return firstSmallUuid + ':' + secondBigUuid;
    };

}
