var serviceModule = angular.module('icu', []);
serviceModule.factory('mapService', function() {

    function changeMyLocation(coords, myUser, map) {
            var myLocation = new Microsoft.Maps.Location(coords.latitude, coords.longitude);
        if (myUser.pin === null) {
            myUser.pin = new Microsoft.Maps.Pushpin(myLocation, { icon: 'images/pin_1_24x36.png' });
            map.entities.push(myUser.pin);
        } else {
            myUser.pin.setLocation(myLocation);
        }

        map.setView({center: myLocation});
        myUser.latitude = coords.latitude;
        myUser.longitude = coords.longitude;
    }

    function addSingleUserToMap(map, user) {
        var userLocation = new Microsoft.Maps.Location(user.ll.lat, user.ll.lng);
        user.pin = new Microsoft.Maps.Pushpin(userLocation, {
            icon: 'images/pin_2_24x36.png'
        });

        //Microsoft.Maps.Events.addHandler(userPin, 'click', startChatOnClick);
        map.entities.push(user.pin);
    }

    function redrawRadiusCircle(map, location, radius) {
        map.entities.remove(map.circlePolygon); // clear circle
        try {
            var latitude = location.latitude;
            var longitude = location.longitude;
            var R = 6371; // earth's mean radius in km
            var lat = (latitude * Math.PI) / 180; //rad
            var lon = (longitude * Math.PI) / 180; //rad
            var d = parseFloat(radius) / R; // d = angular distance covered on earth's surface
            var circlePoints = [];
            for (var x = 0; x <= 360; x++) {
                var p2 = new Microsoft.Maps.Location(0, 0);
                var brng = x * Math.PI / 180; //rad
                p2.latitude  = Math.asin(Math.sin(lat) * Math.cos(d) + Math.cos(lat) * Math.sin(d) * Math.cos(brng));
                p2.longitude = ((lon + Math.atan2(Math.sin(brng) * Math.sin(d) * Math.cos(lat),
                    Math.cos(d) - Math.sin(lat) * Math.sin(p2.latitude ))) * 180) / Math.PI;
                p2.latitude  = (p2.latitude  * 180) / Math.PI;
                circlePoints.push(p2);
            }

            var polygoncolor = new Microsoft.Maps.Color(100, 100, 0, 100);
            map.circlePolygon = new Microsoft.Maps.Polygon(circlePoints, { fillColor: polygoncolor, strokeColor: polygoncolor });

            // Add the polygon to the map
            map.entities.push(map.circlePolygon);

            // Center the map on the location
            //map.setView({center: new Microsoft.Maps.Location(latitude, longitude)});

        } catch(e) {
            // Catch any errors
            console.log("Broken = " + e);
        }
    }

    function getPinLocations(map) {
        var pinLocations = [];
        for (var i = 0; i < map.entities.getLength(); i++) {
            var entity = map.entities.get(i);
            if (entity !== undefined && entity._icon !== undefined && entity._icon.indexOf('pin') !== -1) {
                pinLocations.push(entity.getLocation());
            }
        }

        return pinLocations;
    }

    /**
     * This will only work properly when map is displayed (aka we're on #map view of jQuery mobile page)
     * @param map
     */
    function setBounds(map) {
        var pinLocations = getPinLocations(map);
        var viewBounds = Microsoft.Maps.LocationRect.fromLocations(pinLocations);
        map.setView({bounds: viewBounds});
    }

    // service methods here
    return {
        initMap: function(myUser) {
            var mapOptions = {
                credentials: "Ak9lkQBwprXHnXkunNGh7jda4mIEkuzOzhN15e0RsRnJeRFWXclYPnkcyX519kvM",
                zoom: 13,
                showDashboard: false,
                showMapTypeSelector: false,
                disableBirdseye: true,
                showScalebar: false
            };

            var map = new Microsoft.Maps.Map(document.getElementById("map_canvas"), mapOptions);

            // success and error callbacks
            watchLocation(function(coords) {
                changeMyLocation(coords, myUser, map);
            }, function() {
                console.log('Error: The Geolocation service failed.');
            });

            return map;
        },

        addUsersToMap: function(map, myUser, users, radius) {
            this.clearUserPins(map);
            for (var i in users) {
                addSingleUserToMap(map, users[i]);
            }
            // i get radius in m, but need to convert in km
            redrawRadiusCircle(map, myUser.pin.getLocation(), radius / 1000);
            setBounds(map);
        },

        clearUserPins: function(map) {
            for (var i = 0; i < map.entities.getLength(); i++) {
                var entity = map.entities.get(i);
                if (entity !== undefined && entity._icon !== undefined && entity._icon.indexOf('pin_2') !== -1) {
                    map.entities.remove(entity);
                }
            }
        },

        clearCircle: function(map) {
            map.entities.remove(map.circlePolygon);
        }
    };
});


/**
 * Watch user's location and execute success or error callback
 * @param successCallback
 * @param errorCallback
 */
function watchLocation(successCallback, errorCallback) {
    successCallback = successCallback || function () {};
    errorCallback = errorCallback || function () {};

    var geolocation = navigator.geolocation;
    if (geolocation) {
        try {
            function handleSuccess(position) {
                successCallback(position.coords);
            }
            geolocation.watchPosition(handleSuccess, errorCallback, {
                maximumAge: 3000,
                timeout: 5000,
                enableHighAccuracy: true
            });
        } catch (err) {
            errorCallback();
        }
    } else {
        errorCallback();
    }
}
